/*
 * Copyright (C) 2025 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.admin.reporting;

import static de.ozgcloud.admin.common.errorhandling.ValidationMessageCodes.*;

import java.util.List;

import jakarta.validation.Valid;
import jakarta.validation.constraints.NotBlank;
import jakarta.validation.constraints.NotEmpty;

import org.springframework.data.annotation.Id;
import org.springframework.data.annotation.TypeAlias;
import org.springframework.data.mongodb.core.mapping.Document;

import com.fasterxml.jackson.annotation.JsonIgnore;

import lombok.Builder;
import lombok.Getter;
import lombok.Singular;
import lombok.ToString;
import lombok.extern.jackson.Jacksonized;

@Document(AggregationMapping.COLLECTION_NAME)
@TypeAlias(AggregationMapping.TYPE_ALIAS)
@Jacksonized
@Builder
@Getter
@ToString
public class AggregationMapping {
	static final String COLLECTION_NAME = "aggregationMapping";

	public static final String TYPE_ALIAS = "AggregationMapping";

	@Id
	@JsonIgnore
	private String id;

	@NotEmpty(message = FIELD_IS_EMPTY)
	private String name;

	@Valid
	private FormIdentifier formIdentifier;

	@NotEmpty(message = FIELD_IS_EMPTY)
	@Singular
	@Valid
	private List<FieldMapping> mappings;

	@Builder
	@Getter
	@ToString
	static class FieldMapping {
		@NotBlank(message = FIELD_IS_EMPTY)
		private String sourcePath;
		@NotBlank(message = FIELD_IS_EMPTY)
		private String targetPath;
	}

	@Builder
	@Getter
	@ToString
	static class FormIdentifier {
		@NotBlank(message = FIELD_IS_EMPTY)
		private String formEngineName;
		@NotBlank(message = FIELD_IS_EMPTY)
		private String formId;
	}
}
